import assert from 'assert'
import { Data } from '../app/data'

export class ItemData extends Data {
    uniqKey(): string {
        return 'a'
    }

    a = 0
    b: number
    c: { d: number; e?: number } = { d: 1 }
    __f = 0
    sum() {
        return this.a + this.c.d + this.c.e
    }
}

describe('Data', () => {
    describe('#toObject', () => {
        it('should return object', () => {
            const data = new ItemData()
            data.a = 1
            data.c = { d: 2, e: undefined }
            const object = data.toObject()

            assert.strictEqual('a' in object, true)
            assert.strictEqual('b' in object, false)
            assert.strictEqual('c' in object, true)
            assert.strictEqual('d' in object['c'], true)
            assert.strictEqual('e' in object['c'], true)
            assert.strictEqual('__f' in object, false)
            assert.strictEqual('sum' in object, false)
        })
    })
    describe('#fromObject', () => {
        it('should return data object', () => {
            const dataOrig = new ItemData()
            dataOrig.a = 1
            dataOrig.c = { d: 2, e: undefined }
            const object = dataOrig.toObject()

            const dataRecovered = new ItemData()
            dataRecovered.fromObject(object)

            assert.notStrictEqual(dataOrig, dataRecovered)
        })
    })
})
